/** The NMOS6510 class emulates the NMOS 6510 microprocessor. <BR>
    This code is GPL.
    Copyright (c) 2000 Brice Fines.<DL>
    <UR>
    <LI>This program is free software; you can redistribute it and/or modify<BR>
    it under the terms of the GNU General Public License as published by<BR>
    the Free Software Foundation; either version 2 of the License, or<BR>
    (at your option) any later version.<BR>
    <LI>This program is distributed in the hope that it will be useful,<BR>
    but WITHOUT ANY WARRANTY; without even the implied warranty of<BR>
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<BR>
    GNU General Public License for more details.<BR>
    <LI>You should have received a copy of the GNU General Public License<BR>
    along with this program; if not, write to the Free Software<BR>
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA<BR>
    </UR></DL>
    @version 0.2005 06/06/00
    @author <A HREF="mailto:bfines@uni2.es">Jarod CANAL</A>
    @see "The Gnu Public License." */
public abstract class NMOS6510 extends NMOS6502{

    /** Constructor */
    public NMOS6510(){
    }

    /** Logical and, set carry from bit of A. */
    final private void ANC(){
	processorStatus.set( processorStatus.get() & ~F_C );
	accumulator.set( accumulator.get() & tmp );
	if( ( accumulator.get() & 0x80 ) != 0 )
	    {
		processorStatus.set( processorStatus.get() | F_C );
	    }
	setNZ( accumulator.get() );				    
    }

    /** Logical and, logical shift right. */
    final private void ASR(){
	tmp &= accumulator.get();
	LSR();
    }
    
    /** And stack. <BR>
	Transfer to accumulator and index X logical and stack (LSB) with data, transfer result to S transfer result to accumulator and index X also. */
    final private void AST(){
	stackPointer.setLowByte( stackPointer.getLowByte() & tmp );
	xIndexRegister.set( stackPointer.getLowByte() );
	accumulator.set( xIndexRegister.get() );
	setNZ( accumulator.get() );
    }

    /** Logical and, rotate right. */
    final private void ARR(){
	if( ( processorStatus.get() & F_D ) != 0 )
	    {
		final int lo, hi, t;
		tmp &= accumulator.get();
		t = tmp;
		hi = tmp & 0xF0;
		lo = tmp & 0x0F;
		if( ( processorStatus.get()  & F_C ) != 0 )
		    {
			tmp = ( tmp >> 1 ) | 0x80;
			processorStatus.set( processorStatus.get() | F_N );
		    }
		else
		    {
			tmp >>= 1;
			processorStatus.set( processorStatus.get() & ~F_N );
		    }
		if( tmp != 0 )
		    {
			processorStatus.set( processorStatus.get() & ~F_Z );
		    }
		else
		    {
			processorStatus.set( processorStatus.get() | F_Z );
		    }
		if( ( ( t ^ tmp ) & 0x40 ) != 0 )
		    {
			processorStatus.set( processorStatus.get() | F_V );
		    }
		else
		    {
			processorStatus.set( processorStatus.get() & ~F_V );
		    }
		if( lo + ( lo & 1 ) > 5 )
		    {
			tmp = ( tmp & 0xF0 ) | ( ( tmp + 6 ) & 0xF );
		    }
		if( hi + ( hi & 0x10 ) > 0x50 )
		    {
			processorStatus.set( processorStatus.get() | F_C );
			tmp = ( tmp + 0x60 ) & 0xFF;
		    }
		else
		    {
			processorStatus.set( processorStatus.get() & ~F_C );
		    }
	    }
	else
	    {
		tmp &= accumulator.get();
		ROR();
		processorStatus.set( processorStatus.get() & ~( F_V | F_C ) );
		if ( ( tmp & 0x40 ) != 0 )
		    {
			processorStatus.set( processorStatus.get() | F_C );
		    }
		if ( ( ( tmp & 0x60 ) == 0x20 ) || ( ( tmp & 0x60 ) == 0x40 ) )
		    {
			processorStatus.set( processorStatus.get() | F_V );
		    }
	    }
	
    }

    /** Logical and X w/ A, subtract data from X. */
    final private void ASX(){
	processorStatus.set( processorStatus.get() & ~F_C );
	xIndexRegister.set( xIndexRegister.get() & accumulator.get() );
	if ( xIndexRegister.get() >= tmp )
	    {
		processorStatus.set( processorStatus.get() | F_C );
	    }
	xIndexRegister.set( xIndexRegister.get() - tmp );
	setNZ( xIndexRegister.get() );
    }

    /** Transfer index X to accumulator, logical and. */
    final private void AXA(){
	accumulator.set( ( accumulator.get() | 0xEE ) & xIndexRegister.get() & tmp );
	setNZ( accumulator.get() );
    }

    /** decrement data and compare. */
    final private void DCP(){
	tmp = ( -- tmp ) & 0x00FF;
	processorStatus.set( processorStatus.get() & ~F_C );
	if( accumulator.get() >= tmp )
	    {
		processorStatus.set( processorStatus.get() | F_C );
	    }
	setNZ( ( accumulator.get() - tmp ) & 0x00FF );
    }

    /** Double no operation. */
    final private void DOP(){
	programCounter.setWord( programCounter.getWord() + 1 );
    }

    /** Increment and subtract with carry. */
    final private void ISB(){
	tmp = ( ++ tmp ) & 0x00FF;
	SBC();
    }

    /** Illegal opcode. <BR>
	Processor halted: no hardware interrupt will help, only reset. */
    final private void KIL(){
	programCounter.setWord( programCounter.getWord() - 1 );
    }

    /** Load accumulator and index X. */
    final private void LAX(){
	xIndexRegister.set( tmp );
	accumulator.set( xIndexRegister.get() );
	setNZ( accumulator.get() );
    }

    /** Load accumulator and index X. */
    final private void OAL(){
	xIndexRegister.set( ( accumulator.get() | 0xEE ) & tmp );
	accumulator.set( xIndexRegister.get() );
	setNZ( accumulator.get() );
    }

    /** Rotate left and logical and accumulator. <BR>
	new C <- [7][6][5][4][3][2][1][0] <- C. */
    final private void RLA(){
	tmp = ( tmp << 1 ) | ( processorStatus.get() & F_C );
	processorStatus.set( ( processorStatus.get() & ~F_C ) | ( ( tmp >> 8 ) & F_C ) );
	tmp &= 0x00FF;
	accumulator.set( accumulator.get() & tmp );
	setNZ( accumulator.get() );
    }

    /** Rotate right and add with carry. <BR>
	C -> [7][6][5][4][3][2][1][0] -> C. */
    final private void RRA(){
	tmp |= ( processorStatus.get() & F_C ) << 8;
	processorStatus.set( ( processorStatus.get() & ~F_C ) | ( tmp & F_C ) );
	tmp = ( tmp >> 1 ) & 0x00FF;
	ADC();
    }

    /** Logical and accumulator with index X and store. */
    final private void SAX(){
	tmp = accumulator.get() & xIndexRegister.get();
    }

    /** Shift left and logical or. */
    final private void SLO(){
	processorStatus.set( ( processorStatus.get() & ~F_C ) | ( ( tmp >> 7 ) & F_C ) );
	tmp = ( tmp << 1 ) & 0x00FF;
	accumulator.set( accumulator.get() | tmp );
	setNZ( accumulator.get() );
    }

    /** Logical shift right and logical exclusive or. <BR>
	0 -> [7][6][5][4][3][2][1][0] -> C. */
    final private void SRE(){
	processorStatus.set( ( processorStatus.get() & ~F_C ) | ( tmp & F_C ) );
	tmp = ( tmp >> 1 ) & 0x00FF;
	accumulator.set( accumulator.get() ^ tmp );
	setNZ( accumulator.get() );
    }

    /** Store accumulator and index X and high + 1. <BR>
	Result = accumulator and index X and memory [PC+1] + 1. */
    final private void SAH(){
	tmp = accumulator.get() & xIndexRegister.get() & ( effectiveAddress.getHighByte() + 1 );
    }

    /** Store stack high. <BR>
	Logical and accumulator with index X, transfer result to S.<BR>
	Logical and result with memory [PC+1] + 1. */
    final private void SSH(){
	stackPointer.setLowByte( accumulator.get() & xIndexRegister.get() );
	tmp = stackPointer.getLowByte() & ( effectiveAddress.getHighByte() + 1 );
    }

    /** Store index X high. <BR>
	Logical and index X with memory[PC+1] and store the result. */
    final private void SXH(){
	tmp = xIndexRegister.get() & ( effectiveAddress.getHighByte() + 1 );
    }

    /** Store index Y and (high + 1). <BR>
	Logical and index Y with memory[PC+1] + 1 and store the result. */
    final private void SYH(){
	tmp = yIndexRegister.get() & ( effectiveAddress.getHighByte() + 1 );
    }

    /** Triple no operation. */
    final private void TOP(){
	programCounter.setWord( programCounter.getWord() + 2 );
    }

    /** Opcode 0x80. <BR>
	2 DOP. */
    final private void opcode0x80(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x02. <BR>
	2 KIL. */
    final private void opcode0x02(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x22. <BR>
	2 KIL. */
    final private void opcode0x22(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x42. <BR>
	2 KIL. */
    final private void opcode0x42(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x62. <BR>
	2 KIL. */
    final private void opcode0x62(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x82. <BR>
	2 DOP. */
    final private void opcode0x82(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0xC2. <BR>
	2 DOP. */
    final private void opcode0xC2(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0xE2. <BR>
	2 DOP. */
    final private void opcode0xE2(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x12. <BR>
	2 KIL. */
    final private void opcode0x12(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x32. <BR>
	2 KIL. */
    final private void opcode0x32(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x52. <BR>
	2 KIL. */
    final private void opcode0x52(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x72. <BR>
	2 KIL. */
    final private void opcode0x72(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x92. <BR>
	2 KIL. */
    final private void opcode0x92(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0xB2. <BR>
	2 KIL. */
    final private void opcode0xB2(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0xD2. <BR>
	2 KIL. */
    final private void opcode0xD2(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0xF2. <BR>
	2 KIL. */
    final private void opcode0xF2(){
	iCount -= 2;		 KIL();	// 2 KIL	  
    }

    /** Opcode 0x03. <BR>
	7 SLO IDX. */
    final private void opcode0x03(){
	tmp = 0;	iCount -= 7; RD_IDX(); SLO(); WB_EA(); // 7 SLO IDX
    }

    /** Opcode 0x23. <BR>
	7 RLA IDX. */
    final private void opcode0x23(){
	tmp = 0;	iCount -= 7; RD_IDX(); RLA(); WB_EA(); // 7 RLA IDX
    }

    /** Opcode 0x43. <BR>
	7 SRE IDX. */
    final private void opcode0x43(){
	tmp = 0;	iCount -= 7; RD_IDX(); SRE(); WB_EA(); // 7 SRE IDX
    }

    /** Opcode 0x63. <BR>
	7 RRA IDX. */
    final private void opcode0x63(){
	tmp = 0;	iCount -= 7; RD_IDX(); RRA(); WB_EA(); // 7 RRA IDX
    }

    /** Opcode 0x83. <BR>
	6 SAX IDX. */
    final private void opcode0x83(){
	tmp = 0;	iCount -= 6;           SAX(); WR_IDX(); // 6 SAX IDX
    }	

    /** Opcode 0xA3. <BR>
	6 LAX IDX. */
    final private void opcode0xA3(){
	tmp = 0;	iCount -= 6; RD_IDX(); LAX(); // 6 LAX IDX
    }

    /** Opcode 0xC3. <BR>
	7 DCP IDX. */
    final private void opcode0xC3(){
	tmp = 0;	iCount -= 7; RD_IDX(); DCP(); WB_EA(); // 7 DCP IDX
    }

    /** Opcode 0xE3. <BR>
	7 ISB IDX. */
    final private void opcode0xE3(){
	tmp = 0;	iCount -= 7; RD_IDX(); ISB(); WB_EA(); // 7 ISB IDX
    }

    /** Opcode 0x13. <BR>
	5 SLO IDY. */
    final private void opcode0x13(){
	tmp = 0;	iCount -= 6; RD_IDY(); SLO(); WB_EA(); // 6 SLO IDY
    }

    /** Opcode 0x33. <BR>
	5 RLA IDY. */
    final private void opcode0x33(){
	tmp = 0;	iCount -= 6; RD_IDY(); RLA(); WB_EA(); // 6 RLA IDY
    }

    /** Opcode 0x53. <BR>
	5 SRE IDY. */
    final private void opcode0x53(){
	tmp = 0;	iCount -= 6; RD_IDY(); SRE(); WB_EA(); // 6 SRE IDY
    }

    /** Opcode 0x73. <BR>
	5 RRA IDY. */
    final private void opcode0x73(){
	tmp = 0;	iCount -= 6; RD_IDY(); RRA(); WB_EA(); // 6 RRA IDY
    }

    /** Opcode 0x93. <BR>
	5 SAX IDY. */
    final private void opcode0x93(){
	tmp = 0;	iCount -= 5;           SAH(); WR_IDY(); // 5 SAH IDY
    }	

    /** Opcode 0xB3. <BR>
	5 LAX IDY. */
    final private void opcode0xB3(){
	tmp = 0;	iCount -= 5; RD_IDY(); LAX(); // 5 LAX IDY
    }

    /** Opcode 0xD3. <BR>
	6 DCP IDY. */
    final private void opcode0xD3(){
	tmp = 0;	iCount -= 6; RD_IDY(); DCP(); WB_EA(); // 6 DCP IDY
    }

    /** Opcode 0xF3. <BR>
	6 ISB IDX. */
    final private void opcode0xF3(){
	tmp = 0;	iCount -= 6; RD_IDY(); ISB(); WB_EA(); // 6 ISB IDY
    }

    /** Opcode 0x04. <BR>
	2 DOP. */
    final private void opcode0x04(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x44. <BR>
	2 DOP. */
    final private void opcode0x44(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x64. <BR>
	2 DOP. */
    final private void opcode0x64(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x14. <BR>
	2 DOP. */
    final private void opcode0x14(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x34. <BR>
	2 DOP. */
    final private void opcode0x34(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x54. <BR>
	2 DOP. */
    final private void opcode0x54(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x74. <BR>
	2 DOP. */
    final private void opcode0x74(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0xD4. <BR>
	2 DOP. */
    final private void opcode0xD4(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0xF4. <BR>
	2 DOP. */
    final private void opcode0xF4(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x07. <BR>
	5 SLO ZPG. */
    final private void opcode0x07(){
	tmp = 0;	iCount -= 5; RD_ZPG(); SLO(); WB_EA(); // 5 SLO ZPG
    }

    /** Opcode 0x27. <BR>
	5 RLA ZPG. */
    final private void opcode0x27(){
	tmp = 0;	iCount -= 5; RD_ZPG(); RLA(); WB_EA(); // 5 RLA ZPG
    }

    /** Opcode 0x47. <BR>
	5 SRE ZPG. */
    final private void opcode0x47(){
	tmp = 0;	iCount -= 5; RD_ZPG(); SRE(); WB_EA(); // 5 SRE ZPG
    }

    /** Opcode 0x67. <BR>
	5 RRA ZPG. */
    final private void opcode0x67(){
	tmp = 0;	iCount -= 5; RD_ZPG(); RRA(); WB_EA(); // 5 RRA ZPG
    }

    /** Opcode 0x87. <BR>
	3 SAX ZPG. */
    final private void opcode0x87(){
	tmp = 0;	iCount -= 3;           SAX(); WR_ZPG(); // 3 SAX ZPG
    }	

    /** Opcode 0xA7. <BR>
	3 LAX ZPG. */
    final private void opcode0xA7(){
	tmp = 0;	iCount -= 3; RD_ZPG(); LAX(); // 3 LAX ZPG
    }

    /** Opcode 0xC7. <BR>
	5 DCP ZPG. */
    final private void opcode0xC7(){
	tmp = 0;	iCount -= 5; RD_ZPG(); DCP(); WB_EA(); // 5 DCP ZPG
    }

    /** Opcode 0xE7. <BR>
	5 ISB ZPG. */
    final private void opcode0xE7(){
	tmp = 0;	iCount -= 5; RD_ZPG(); ISB(); WB_EA(); // 5 ISB ZPG
    }

    /** Opcode 0x17. <BR>
	6 SLO ZPX. */
    final private void opcode0x17(){
	tmp = 0;	iCount -= 6; RD_ZPX(); SLO(); WB_EA(); // 6 SLO ZPX
    }

    /** Opcode 0x37. <BR>
	6 RLA ZPX. */
    final private void opcode0x37(){
	tmp = 0;	iCount -= 6; RD_ZPX(); RLA(); WB_EA(); // 6 RLA ZPX
    }

    /** Opcode 0x57. <BR>
	6 SRE ZPX. */
    final private void opcode0x57(){
	tmp = 0;	iCount -= 6; RD_ZPX(); SRE(); WB_EA(); // 6 SRE ZPX
    }

    /** Opcode 0x77. <BR>
	6 RRA ZPX. */
    final private void opcode0x77(){
	tmp = 0;	iCount -= 6; RD_ZPX(); RRA(); WB_EA(); // 6 RRA ZPX
    }

    /** Opcode 0x97. <BR>
	4 SAX ZPX. */
    final private void opcode0x97(){
	tmp = 0;	iCount -= 4;           SAX(); WR_ZPY(); // 4 SAX ZPY
    }	

    /** Opcode 0xB7. <BR>
	4 LAX ZPX. */
    final private void opcode0xB7(){
	tmp = 0;	iCount -= 4; RD_ZPY(); LAX(); // 4 LAX ZPY
    }

    /** Opcode 0xD7. <BR>
	6 DCP ZPX. */
    final private void opcode0xD7(){
	tmp = 0;	iCount -= 6; RD_ZPX(); DCP(); WB_EA(); // 6 DCP ZPX
    }

    /** Opcode 0xF7. <BR>
	6 ISB ZPX. */
    final private void opcode0xF7(){
	tmp = 0;	iCount -= 6; RD_ZPX(); ISB(); WB_EA(); // 6 ISB ZPX
    }

    /** Opcode 0x89. <BR>
	2 DOP. */
    final private void opcode0x89(){
	iCount -= 2;		 DOP();	// 2 DOP	  
    }

    /** Opcode 0x1A. <BR>
	2 NOP. */
    final private void opcode0x1A(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0x3A. <BR>
	2 NOP. */
    final private void opcode0x3A(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0x5A. <BR>
	2 NOP. */
    final private void opcode0x5A(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0x7A. <BR>
	2 NOP. */
    final private void opcode0x7A(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0xDA. <BR>
	2 NOP. */
    final private void opcode0xDA(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0xFA. <BR>
	2 NOP. */
    final private void opcode0xFA(){
	iCount -= 2;		 NOP();	// 2 NOP	  
    }

    /** Opcode 0x0B. <BR>
	2 ANC IMM. */
    final private void opcode0x0B(){
	tmp = 0;	iCount -= 2; RD_IMM(); ANC(); // 2 ANC IMM
    }

    /** Opcode 0x2B. <BR>
	2 ANC IMM. */
    final private void opcode0x2B(){
	tmp = 0;	iCount -= 2; RD_IMM(); ANC(); // 2 ANC IMM
    }

    /** Opcode 0x4B. <BR>
	2 ASR IMM. */
    final private void opcode0x4B(){
	tmp = 0;	iCount -= 2; RD_IMM(); ASR(); WB_ACC();  // 2 ASR IMM
    }

    /** Opcode 0x6B. <BR>
	2 ARR IMM. */
    final private void opcode0x6B(){
	tmp = 0;	iCount -= 2; RD_IMM(); ARR(); WB_ACC();  // 2 ARR IMM
    }

    /** Opcode 0x8B. <BR>
	2 AXAIMM. */
    final private void opcode0x8B(){
	tmp = 0;	iCount -= 2; RD_IMM(); AXA();  // 2 AXA IMM
    }

    /** Opcode 0xAB. <BR>
	2 OAL IMM. */
    final private void opcode0xAB(){
	tmp = 0;	iCount -= 2; RD_IMM(); OAL(); // 2 OAL IMM
    }

    /** Opcode 0xCB. <BR>
	2 ASX IMM. */
    final private void opcode0xCB(){
	tmp = 0;	iCount -= 2; RD_IMM(); ASX(); // 2 ASX IMM
    }

    /** Opcode 0xEB. <BR>
	2 SBC IMM. */
    final private void opcode0xEB(){
	tmp = 0;	iCount -= 2; RD_IMM(); SBC(); // 2 SBC IMM
    }

    /** Opcode 0x1B. <BR>
	4 SLO ABY. */
    final private void opcode0x1B(){
	tmp = 0;	iCount -= 4; RD_ABY(); SLO(); WB_EA(); // 4 SLO ABY
    }

    /** Opcode 0x3B. <BR>
	4 RLA ABY. */
    final private void opcode0x3B(){
	tmp = 0;	iCount -= 4; RD_ABY(); RLA(); WB_EA(); // 4 RLA ABY
    }

    /** Opcode 0x5B. <BR>
	4 SRE ABY. */
    final private void opcode0x5B(){
	tmp = 0;	iCount -= 4; RD_ABY(); SRE(); WB_EA(); // 4 SRE ABY
    }

    /** Opcode 0x7B. <BR>
	4 RRA ABY. */
    final private void opcode0x7B(){
	tmp = 0;	iCount -= 4; RD_ABY(); RRA(); SRE(); WB_EA(); // 4 RRA ABY
    }

    /** Opcode 0x9B. <BR>
	5 SSH ABY. */
    final private void opcode0x9B(){
	tmp = 0;	iCount -= 5; EA_ABY(); SSH(); WB_EA(); // 5 SSH ABY
    }

    /** Opcode 0xBB. <BR>
	4 AST ABY. */
    final private void opcode0xBB(){
	tmp = 0;	iCount -= 4; RD_ABY(); AST(); // 4 AST ABY
    }

    /** Opcode 0xDB. <BR>
	6 SLO ABY. */
    final private void opcode0xDB(){
	tmp = 0;	iCount -= 6; RD_ABY(); DCP(); WB_EA(); // 6 DCP ABY
    }

    /** Opcode 0xFB. <BR>
	6 SLO ABY. */
    final private void opcode0xFB(){
	tmp = 0;	iCount -= 6; RD_ABY(); ISB(); WB_EA(); // 6 ISB ABY
    }

    /** Opcode 0x0C. <BR>
	2 TOP. */
    final private void opcode0x0C(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x1C. <BR>
	2 TOP. */
    final private void opcode0x1C(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x3C. <BR>
	2 TOP. */
    final private void opcode0x3C(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x5C. <BR>
	2 TOP. */
    final private void opcode0x5C(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x7C. <BR>
	2 TOP. */
    final private void opcode0x7C(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x9C. <BR>
	5 SYH ABX. */
    final private void opcode0x9C(){
	tmp = 0;	iCount -= 5; EA_ABX(); SYH(); WB_EA();	// 5 SYH ABX	  
    }

    /** Opcode 0xDC. <BR>
	2 TOP. */
    final private void opcode0xDC(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0xFC. <BR>
	2 TOP. */
    final private void opcode0xFC(){
	iCount -= 2;		 TOP();	// 2 TOP	  
    }

    /** Opcode 0x9E. <BR>
	2 SXH ABY. */
    final private void opcode0x9E(){
	tmp = 0;	iCount -= 2; EA_ABY(); SXH(); WB_EA(); // 2 SXH ABY
    }

    /** Opcode 0x0F. <BR>
	6 SLO ABS. */
    final private void opcode0x0F(){
	tmp = 0;	iCount -= 6; RD_ABS(); SLO(); WB_EA(); // 6 SLO ABS
    }

    /** Opcode 0x2F. <BR>
	6 RLA ABS. */
    final private void opcode0x2F(){
	tmp = 0;	iCount -= 6; RD_ABS(); RLA(); WB_EA(); // 6 RLA ABS
    }

    /** Opcode 0x4F. <BR>
	6 SRE ABS. */
    final private void opcode0x4F(){
	tmp = 0;	iCount -= 6; RD_ABS(); SRE(); WB_EA(); // 6 SRE ABS
    }

    /** Opcode 0x6F. <BR>
	6 RRA ABS. */
    final private void opcode0x6F(){
	tmp = 0;	iCount -= 6; RD_ABS(); RRA(); WB_EA(); // 6 RRA ABS
    }

    /** Opcode 0x8F. <BR>
	4 SAX ABS. */
    final private void opcode0x8F(){
	tmp = 0;	iCount -= 4;           SAX(); WR_ABS(); // 4 SAX ABS
    }	

    /** Opcode 0xAF. <BR>
	5 LAX ABS. */
    final private void opcode0xAF(){
	tmp = 0;	iCount -= 5; RD_ABS(); LAX(); // 5 LAX ABS
    }

    /** Opcode 0xCF. <BR>
	6 DCP ABS. */
    final private void opcode0xCF(){
	tmp = 0;	iCount -= 6; RD_ABS(); DCP(); WB_EA(); // 6 DCP ABS
    }

    /** Opcode 0xEF. <BR>
	6 ISB ABS. */
    final private void opcode0xEF(){
	tmp = 0;	iCount -= 6; RD_ABS(); ISB(); WB_EA(); // 6 ISB ABS
    }

    /** Opcode 0x1F. <BR>
	4 SLO ABX. */
    final private void opcode0x1F(){
	tmp = 0;	iCount -= 4; RD_ABX(); SLO(); WB_EA(); // 4 SLO ABX
    }

    /** Opcode 0x3F. <BR>
	4 RLA ABX. */
    final private void opcode0x3F(){
	tmp = 0;	iCount -= 4; RD_ABX(); RLA(); WB_EA(); // 4 RLA ABX
    }

    /** Opcode 0x5F. <BR>
	4 SRE ABX. */
    final private void opcode0x5F(){
	tmp = 0;	iCount -= 4; RD_ABX(); SRE(); WB_EA(); // 4 SRE ABX
    }

    /** Opcode 0x7F. <BR>
	4 RRA ABX. */
    final private void opcode0x7F(){
	tmp = 0;	iCount -= 4; RD_ABX(); RRA(); WB_EA(); // 4 RRA ABX
    }

    /** Opcode 0x9F. <BR>
	6 SAH ABY. */
    final private void opcode0x9F(){
	tmp = 0;	iCount -= 6; EA_ABY(); SAH(); WB_EA(); // 6 SAH ABY
    }	

    /** Opcode 0xBF. <BR>
	6 LAX ABY. */
    final private void opcode0xBF(){
	tmp = 0;	iCount -= 6; RD_ABY(); LAX(); // 6 LAX ABY
    }

    /** Opcode 0xDF. <BR>
	7 DCP ABX. */
    final private void opcode0xDF(){
	tmp = 0;	iCount -= 7; RD_ABX(); DCP(); WB_EA(); // 7 DCP ABX
    }

    /** Opcode 0xFF. <BR>
	7 ISB ABX. */
    final private void opcode0xFF(){
	tmp = 0;	iCount -= 7; RD_ABX(); ISB(); WB_EA(); // 7 ISB ABX
    }
}
